@@ -4,7 +4,7 @@ PIXIU API
 from typing import NewType, TypeVar, Generic
 import abc
 from datetime import datetime
-from pixiu.api.defines import (TimeFrame, OrderType, OrderCommand)
+from pixiu.api.defines import (TimeFrame, OrderType, OrderCommand, RunModeValue)
 from .order import Order
 
 class IndicatiorID():
@@ -27,11 +27,6 @@ class IndicatiorID():
     TEMA = 17000
     WPR = 18000
 
-class OrderScope():
-    EA = 0
-    EA_VERSION = 200
-    ACCOUNT = 300
-
 class DataScope():
     EA = 0
     EA_VERSION = 200
@@ -54,6 +49,7 @@ class APIStub(abc.ABC):
         OrderCommand=OrderCommand,
         Order=Order,
         DataScope=DataScope,
+        RunModeValue=RunModeValue,
     )
 
     def __init__(self):
@@ -71,7 +67,7 @@ class APIStub(abc.ABC):
             magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None,
             tags=None) -> (ErrorID, OrderResult):
         '''
-        Open a buy order.
+        Open a long order.
 
                 Parameters:
                         volume (float): Number of lots.
@@ -84,6 +80,7 @@ class APIStub(abc.ABC):
                         slippage (float): Maximum price slippage for trading.
                         arrow_color (float): Color of the opening arrow on the MT4/5 chart.
                         expiration (float): Order expiration time (for pending order only)
+                        tags (dict): Order tags
                 Returns:
                         ErrorID: If 0 success.
                         OrderResult: The order result.
@@ -95,7 +92,7 @@ class APIStub(abc.ABC):
              magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None,
              tags=None) -> (ErrorID, OrderResult):
         '''
-        Open a sell order.
+        Open a short order.
 
                 Parameters:
                         volume (float): Number of lots.
@@ -108,6 +105,7 @@ class APIStub(abc.ABC):
                         slippage (float): Maximum price slippage for trading.
                         arrow_color (float): Color of the opening arrow on the MT4/5 chart.
                         expiration (float): Order expiration time (for pending order only)
+                        tags (dict): Order tags
                 Returns:
                         ErrorID: If 0 success.
                         OrderResult: The order result.
@@ -127,6 +125,7 @@ class APIStub(abc.ABC):
                         take_profit (float): New take profit price.
                         arrow_color (float): New color of the opening arrow on the MT4/5 chart.
                         expiration (float): New order expiration time (for pending order only)
+                        tags (dict): Order tags
                 Returns:
                         ErrorID: If 0 success.
                         OrderResult: The order result.
@@ -145,6 +144,23 @@ class APIStub(abc.ABC):
                         price (float): Close price. If price is None or zero, price = OrderClosePrice(MT4/MT5)
                         slippage (float): Maximum price slippage for trading.
                         arrow_color (float): New color of the opening arrow on the MT4/5 chart.
+                        tags (dict): Order tags
+                Returns:
+                        ErrorID: If 0 success.
+                        OrderResult: The order result.
+        '''
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def CloseMultiOrders(self, orders) -> (ErrorID, OrderResult):
+        '''
+        Close a order.
+
+                Parameters:
+                        orders : list or dict
+                                 ['uid1', 'uid2', 'uid3' ... ]
+                                 OR
+                                 {'uid1': {'price', 'volume', 'slippage', comment, tags}, ...}
                 Returns:
                         ErrorID: If 0 success.
                         OrderResult: The order result.
@@ -186,7 +202,7 @@ class APIStub(abc.ABC):
         Returns the symbol properties.
 
                 Parameters:
-                        symbol (int): The symbol name.
+                        symbol (str): The symbol name.
 
                 Returns:
                         The symbol properties
@@ -417,7 +433,7 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def GetOpenedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
+    def GetOpenedOrderUIDs(self, symbol: str = None, scope: int = DataScope.EA):
         '''
         Returns the UIDs of current opened orders.
 
@@ -436,7 +452,7 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def GetPendingOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
+    def GetPendingOrderUIDs(self, symbol: str = None, scope: int = DataScope.EA):
         '''
         Returns the UIDs of current pending orders.
 
@@ -455,7 +471,7 @@ class APIStub(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def GetClosedOrderUIDs(self, symbol: str = None, scope: int = OrderScope.EA):
+    def GetClosedOrderUIDs(self, symbol: str = None, scope: int = DataScope.EA):
         '''
         Returns the UIDs of current closed orders.
 
@@ -863,4 +879,24 @@ class APIStub(abc.ABC):
         '''
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def Notify(self, message) -> ErrorID:
+        '''
+        Send a notification
 
+                Parameters:
+                        message: The content of notification
+                Returns:
+                        The errorid.
+        '''
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def RunMode(self) -> str:
+        '''
+        Returns the current run mode.
+
+                Returns:
+                        The run mode. (TEST/LIVE)
+        '''
+        raise NotImplementedError
