@@ -4,8 +4,9 @@ import numpy as np
 import talib
 import logging
 from datetime import datetime
-from pixiu.api.v1 import (TimeFrame, OrderCommand, OrderType, IndicatiorID, SymbolIndicator, SymbolPrice, SymbolData,
-                      Order, Account, Symbol, AccountData, ErrorID, OrderResult)
+from pixiu.api import RunModeValue
+from pixiu.api.v1 import (OrderCommand, OrderType, IndicatiorID, SymbolIndicator, SymbolPrice, SymbolData,
+                      Order, Account, Symbol, AccountData, ErrorID, OrderResult, DataScope)
 log = logging.getLogger(__name__)
 
 
@@ -22,7 +23,7 @@ class TesterAPI_V1(API_V1_Base):
         return self.tester.get_print_factory(*args, **kwargs)
     #
     def Buy(self, volume, type=OrderType.MARKET, price=0, stop_loss=None, take_profit=None, comment=None,
-            magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None):
+            magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None, tags=None):
         if symbol is None:
             symbol = self.default_symbol
         if price <= 0:
@@ -39,10 +40,10 @@ class TesterAPI_V1(API_V1_Base):
 
         return self.tester.open_order(symbol, oct, price, volume, stop_loss, take_profit,
                                            comment=comment, magic_number=magic_number, slippage=slippage,
-                                           arrow_color=arrow_color)
+                                           arrow_color=arrow_color, tags=tags)
 
     def Sell(self, volume, type=OrderType.MARKET, price=0, stop_loss=None, take_profit=None, comment=None,
-             magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None):
+             magic_number=None, symbol=None, slippage=None, arrow_color=None, expiration=None, tags=None):
         if symbol is None:
             symbol = self.default_symbol
         if price <= 0:
@@ -58,21 +59,36 @@ class TesterAPI_V1(API_V1_Base):
             raise NotImplementedError
         return self.tester.open_order(symbol, oct, price, volume, stop_loss, take_profit,
                                            comment=comment, magic_number=magic_number, slippage=slippage,
-                                           arrow_color=arrow_color)
+                                           arrow_color=arrow_color, tags=tags)
 
     def ModifyOrder(self, uid, price=None, stop_loss=None, take_profit=None, comment=None,
-                     arrow_color=None, expiration=None):
+                     arrow_color=None, expiration=None, tags=None):
         return self.tester.modify_order(uid, price, stop_loss, take_profit,
-                              comment=comment, arrow_color=arrow_color, expiration=expiration)
+                              comment=comment, arrow_color=arrow_color, expiration=expiration, tags=tags)
 
     #
-    def CloseOrder(self, uid, volume: float=None, price=None,  slippage=None, arrow_color=None):
+    def CloseOrder(self, uid, volume: float=None, price=None,  slippage=None, arrow_color=None, tags=None):
         return self.tester.close_order(uid, volume, price, slippage=slippage,
-                              arrow_color=arrow_color)
+                              arrow_color=arrow_color, tags=tags)
+
+    def CloseMultiOrders(self, orders):
+        return self.tester.close_multi_orders(orders)
+
+    def Notify(self, message):
+        return self.tester.notify(message)
+
+    def RunMode(self):
+        return RunModeValue.TEST
 
     def WaitCommand(self, uid, timeout=120):
         return self.tester.wait_command(uid, timeout)
 
+    def AcquireLock(self, name, timeout=60):
+        return self.tester.acquire_lock(name, timeout)
+
+    def ReleaseLock(self, name):
+        return self.tester.release_lock(name)
+
     def DefaultTimeFrame(self):
         return self.tester.tick_timeframe
 
@@ -122,51 +138,62 @@ class TesterAPI_V1(API_V1_Base):
                 ret['open_lowest'] = order.open_price
         return ret
 
+    def Plot(self, series):
+        self.tester.plot('default', series)
 
     def GetParam(self, name, default=None):
         '''Get params'''
-        return self.get_param(name, default)
+        return self.tester.get_param(name, default)
+
+    def DeleteData(self, name, scope: int = DataScope.EA):
+        return self.tester.delete_data(name, scope)
+
+    def LoadData(self, name, scope: int = DataScope.EA, format='json'):
+        return self.tester.load_data(name, scope, format)
+
+    def SaveData(self, name, data, scope: int = DataScope.EA, format='json'):
+        return self.tester.save_data(name, data, scope, format)
 
     def GetOrder(self, order_uid: str):
         return Order(self.tester.get_order(order_uid=order_uid))
 
-    def Close(self, shift=0) -> float:
-        return self.tester.Close(shift=shift)
+    def Close(self, shift=0, symbol=None) -> float:
+        return self.tester.Close(shift=shift, symbol=symbol)
 
-    def Open(self, shift=0) -> float:
-        return self.tester.Open(shift=shift)
+    def Open(self, shift=0, symbol=None) -> float:
+        return self.tester.Open(shift=shift, symbol=symbol)
 
-    def High(self, shift=0) -> float:
-        return self.tester.High(shift=shift)
+    def High(self, shift=0, symbol=None) -> float:
+        return self.tester.High(shift=shift, symbol=symbol)
 
-    def Low(self, shift=0) -> float:
-        return self.tester.Low(shift=shift)
+    def Low(self, shift=0, symbol=None) -> float:
+        return self.tester.Low(shift=shift, symbol=symbol)
 
-    def Ask(self, shift=0) -> float:
-        return self.tester.Ask(shift=shift)
+    def Ask(self, shift=0, symbol=None) -> float:
+        return self.tester.Ask(shift=shift, symbol=symbol)
 
-    def Bid(self, shift=0) -> float:
-        return self.tester.Bid(shift=shift)
+    def Bid(self, shift=0, symbol=None) -> float:
+        return self.tester.Bid(shift=shift, symbol=symbol)
 
-    def Time(self, shift=0) -> datetime:
-        return self.tester.Time(shift=shift)
+    def Time(self, shift=0, symbol=None) -> datetime:
+        return self.tester.Time(shift=shift, symbol=symbol)
 
-    def Volume(self, shift=0) -> float:
-        return self.tester.Volume(shift=shift)
+    def Volume(self, shift=0, symbol=None) -> float:
+        return self.tester.Volume(shift=shift, symbol=symbol)
 
     def Symbol(self, shift=0) -> str:
         return self.tester.symbol
 
-    def GetOpenedOrderUIDs(self, symbol: str = None):
-        ret = self.tester.get_order_dict(symbol, status="opened", script_name=None)
+    def GetOpenedOrderUIDs(self, symbol: str = None, scope: int = DataScope.EA):
+        ret = self.tester.get_order_dict(symbol, status="opened", scope=scope)
         return list(ret.keys())
 
-    def GetPendingOrderUIDs(self, symbol: str = None):
-        ret = self.tester.get_order_dict(symbol, status="pending", script_name=None)
+    def GetPendingOrderUIDs(self, symbol: str = None, scope: int = DataScope.EA):
+        ret = self.tester.get_order_dict(symbol, status="pending", scope=scope)
         return list(ret.keys())
 
-    def GetClosedOrderUIDs(self, symbol: str = None):
-        ret = self.tester.get_order_dict(symbol, status="closed", script_name=None)
+    def GetClosedOrderUIDs(self, symbol: str = None, scope: int = DataScope.EA):
+        ret = self.tester.get_order_dict(symbol, status="closed", scope=scope)
         return list(ret.keys())
 
     def StopTester(self, code: int=0, message: str=None):
@@ -427,3 +454,4 @@ class TesterAPI_V1(API_V1_Base):
         '''
         # real = ADX(high, low, close, timeperiod=14)
         return self.__get_indicator__(f"wpr:", shift, IndicatiorID.WPR, symbol_data, timeperiod)
+
